package org.geekfu.MapPanel;

import info.clearthought.layout.TableLayout;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.geekfu.OldCartographer.Map;

public class MapPanel extends JPanel implements ActionListener{

	protected Map map;
	protected JScrollPane mapscroll;
	protected JPanel sidebar;

	protected FireableTextField textentry;
	protected JTextArea textoutput;
	protected JScrollPane textscroll;

	protected JFrame textWindow;
	protected JTextArea windowTextEntry;

	protected JSplitPane textMapSplit,sidebarSplit;

	protected List<MapPanelListener> listeners=new LinkedList<MapPanelListener>();

	private boolean textVisible;
	private boolean sidebarVisible;
	
	private static final String IMAGEPATH = "/org/geekfu/hippo/blueHippo.png";

	/**
	 * @param arg0
	 * @param arg1
	 * @param map
	 * @param sidebar
	 * @param text
	 */
	public MapPanel(Map map, boolean sidebar,boolean text) {
		super();

		init(map,sidebar,text);
	}

	protected MapPanel() {}

	protected void init(Map map) {
		init(map,false,false);
	}

	protected void init(Map map, boolean sidebar,boolean text) {
		this.map = map;
		this.sidebar = new JPanel();
		
		this.sidebarVisible=sidebar;
		this.textVisible=text;

		textentry=new FireableTextField();
		textoutput=new JTextArea();
		textoutput.setEditable(false);
		textscroll = new JScrollPane(textoutput);
		textscroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		layoutWindow();
	}


	private void layoutWindow(){
		JComponent mapComponent=map;

		if(mapComponent==null){
			Image img;
			try {
				img = ImageIO.read(getClass().getResource(IMAGEPATH));
				mapComponent=new ImageComponent(img);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		mapscroll=new JScrollPane(new CenteredPanel(mapComponent));
		if(map!=null){
			mapscroll.getVerticalScrollBar().setUnitIncrement(map.getCellsize().height/2);
			mapscroll.getHorizontalScrollBar().setUnitIncrement(map.getCellsize().width/2);
		}

		if(sidebarVisible){
			if(textVisible){ ///////////////////////////////////// With text widgets
				layoutTextSidebar(this);
			}else { ///////////////////////////////////// Without text widgets
				layoutNoTextSidebar(this);
			}
		}else{
			if(textVisible){ ///////////////////////////////////// With text widgets
				layoutTextNoSidebar(this);
			}else { ///////////////////////////////////// Without text widgets
				layoutNoTextNoSidebar(this);
			}
		}
		validate();
	}

	private void killBorder(JComponent target){
		target.setBorder(BorderFactory.createEmptyBorder());
	}
	
	private void layoutNoTextSidebar(JPanel target) {
		killBorder(target);

		double[][] sizes= {
				{TableLayout.FILL},
				{
					TableLayout.FILL},
		};
		target.setLayout(new TableLayout(sizes));

		sidebarSplit=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		sidebarSplit.setResizeWeight(1.0);

		sidebarSplit.add(mapscroll);
		sidebarSplit.add(sidebar);
		killBorder(sidebarSplit);
		target.add(sidebarSplit,"0, 0");
		int w=target.getWidth();
		if(w==0 && target.getParent()!=null)w=target.getParent().getWidth();
		sidebarSplit.setDividerLocation(w*7/10);
	}

	private void layoutTextSidebar(JPanel target) {
		killBorder(target);

		double[][] sizes= {
				{TableLayout.FILL},
				{
					TableLayout.FILL},
		};
		setLayout(new TableLayout(sizes));

		sidebarSplit=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		sidebarSplit.setResizeWeight(1.0);
		
		JPanel left=new JPanel();
		layoutTextNoSidebar(left);
		sidebarSplit.add(left);
		sidebarSplit.add(sidebar);
		killBorder(sidebarSplit);
		target.add(sidebarSplit,"0, 0");
		int w=target.getWidth();
		if(w==0 && target.getParent()!=null)w=target.getParent().getWidth();
		sidebarSplit.setDividerLocation(w*7/10);
	}

	private void layoutNoTextNoSidebar(JPanel target) {
		killBorder(target);

		double[][] sizes= {
				{TableLayout.FILL},
				{
					TableLayout.FILL},
		};
		target.setLayout(new TableLayout(sizes));
		target.add(mapscroll,"0, 0");
	}

	private void layoutTextNoSidebar(JPanel target) {
		killBorder(target);

		textMapSplit=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		textMapSplit.add(mapscroll);
		textMapSplit.add(textscroll);
		textMapSplit.setDividerLocation(400);
		textMapSplit.setResizeWeight(1.0);

		double[][] sizes= {
				{TableLayout.FILL, TableLayout.PREFERRED},
				{
					TableLayout.FILL, 
					TableLayout.PREFERRED},
		};
		target.setLayout(new TableLayout(sizes));
		killBorder(textMapSplit);

		// Add split pane
		target.add(textMapSplit,"0, 0, 1, 0");

		// Add text entry and button
		target.add(textentry,"0, 1, 0, 1");
		JButton showTextWindow=new JButton("...");
		showTextWindow.addActionListener(this);
		target.add(showTextWindow,"1, 1, 1, 1");
	}


	public void scrollToCell(int x,int y) {
		if(map==null)return;
		Dimension c=map.getCellsize();
		Rectangle r=new Rectangle(x*c.width,y*c.height,c.width,c.height);
		map.scrollRectToVisible(r);
	}


	public void addActionListener(MapPanelListener a) {
		listeners.add(a);
	}

	public void showString(String message){
		if(!textVisible)return;
		textoutput.append(message);
		if(!message.endsWith("\n"))
			textoutput.append("\n");
		textoutput.scrollRectToVisible(new Rectangle(0,textoutput.getHeight(),1,1));
	}

	public Map getMap() {
		return map;
	}

	public JPanel getSidebar() {
		return sidebar;
	}

	public boolean isText() {
		return textVisible;
	}

	public void setMap(Map map) {
		this.map = map;
		layoutWindow();
	}

	public void setSidebar(JPanel sidebar) {
		this.sidebar = sidebar;
		layoutWindow();
	}

	public void setText(boolean text) {
		this.textVisible = text;
		layoutWindow();
	}

	public void actionPerformed(ActionEvent arg0) {
		if(textWindow==null){
			textWindow=new JFrame();
			textWindow.setSize(300,400);
			windowTextEntry=new JTextArea();

			JScrollPane textscroll = new JScrollPane(windowTextEntry);
			textscroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

			textWindow.setLayout(new BorderLayout());

			textWindow.getContentPane().add(textscroll,BorderLayout.CENTER);
			JButton enter=new JButton("Enter");
			enter.addActionListener(new WindowButtonListener());
			textWindow.getContentPane().add(enter,BorderLayout.SOUTH);
		}

		windowTextEntry.setText(textentry.getText());
		textWindow.setVisible(true);
	}

	private class WindowButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent arg0) {
			String str=windowTextEntry.getText();
			textWindow.setVisible(false);
			textentry.setText(str);
			textentry.fireAction();
		}
	}

	private class FireableTextField extends JTextField implements KeyListener,ActionListener{
		private LinkedList<String> values=new LinkedList<String>();
		private int hist=-1;
		private String working="";
		public FireableTextField(){
			addKeyListener(this);
			addActionListener(this);
		}
		public void fireAction(){
			fireActionPerformed();
		}

		public void keyReleased(KeyEvent event) {}
		public void keyTyped(KeyEvent event) {}
		public void keyPressed(KeyEvent event) {
			if(event.getKeyCode()==KeyEvent.VK_UP){
				if(hist==-1){
					working=getText();
				}else{
					values.set(hist, getText());
				}
				if(hist<values.size()-1){
					hist++;
					setText(values.get(hist));
				}
			}else if(event.getKeyCode()==KeyEvent.VK_DOWN){
				if(hist==-1){
					working=getText();
				}
				else{
					values.set(hist, getText());
				}

				if(hist>0){
					hist--;
					setText(values.get(hist));
				}else if(hist>-1){
					hist--;
					setText(working);
				}
			}
		}
		public void actionPerformed(ActionEvent evt) {
			String str=getText();
			values.addFirst(str);
			hist=-1;
			working="";
			setText(working);
			for(MapPanelListener m : listeners){
				m.textEntered(str);
			}
		}
	}

	public JTextField getTextEntry() {
		return textentry;
	}

	public boolean isSidebarVisible() {
		return sidebarVisible;
	}

	public void setSidebarVisible(boolean sidebarVisible) {
		this.sidebarVisible = sidebarVisible;
		layoutWindow();
	}
}