package org.geekfu.hippo;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Stroke;

import org.geekfu.OldCartographer.MapCell;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.Undefined;

public class HippoCell extends MapCell {
	Object details;
	ImageManager im=ImageManager.instance();
	boolean space=false;
	public static final int N=1,S=2,E=4,W=8,NE=16,SE=32,NW=64,SW=128;
	
	public HippoCell(Object details){
		setDetails(details);
	}

	public HippoCell(Object details,boolean space){ // space is true if it's a map space, false for a piece
		this.space=space;
		setDetails(details);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Rectangle d=new Rectangle(getSize());
		if(isColor()){
			Color color=findColor();
			String shape=findShape();
			g.setColor(color);
			if(space){
				g.fillRect(0, 0, d.width, d.height);
			}else{
				if("cross".equals(shape)){
					Stroke str=new BasicStroke(d.width/10,BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
					((Graphics2D)g).setStroke(str);
					int w=d.width/4;
					g.drawLine(w, w, 3*w, 3*w);
					g.drawLine(3*w, w, w, 3*w);
				}else if("square".equals(shape)){
					g.fillRect(d.width/4,d.width/4,d.width/2+1,d.height/2+1);
				}else if("outline".equals(shape)){
					int w=d.width/10;
					Stroke str=new BasicStroke(w,BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER);
					((Graphics2D)g).setStroke(str);
					g.drawRect(w/2,w/2,d.width-w-1,d.height-w-1);
				}else if("lines".equals(shape)){ // nsew from center to edge
					Stroke str=new BasicStroke(d.width/10,BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
					((Graphics2D)g).setStroke(str);
					int sides=getSides();
					int m=d.width/2; // middle
					int e=d.width; // edge
					if((sides&N)>0)g.drawLine(m, m, m, 0);
					if((sides&S)>0)g.drawLine(m, m, m, e);
					if((sides&E)>0)g.drawLine(m, m, e, m);
					if((sides&W)>0)g.drawLine(m, m, 0, m);
					
					if((sides&NE)>0)g.drawLine(m, m, e, 0);
					if((sides&SE)>0)g.drawLine(m, m, e, e);
					if((sides&NW)>0)g.drawLine(m, m, 0, 0);
					if((sides&SW)>0)g.drawLine(m, m, 0, 0);
				}else if("walls".equals(shape)){ // nsew along edge
					BasicStroke str=new BasicStroke(d.width/10,BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND);
					((Graphics2D)g).setStroke(str);
					int sides=getSides();
					int e=(int) (d.width-str.getLineWidth()/2)-1; // edge
					int m=(int) str.getLineWidth()/2; // min
					if((sides&N)>0)g.drawLine(m,m,e,m);
					if((sides&S)>0)g.drawLine(m,e,e,e);
					if((sides&E)>0)g.drawLine(e,m,e,e);
					if((sides&W)>0)g.drawLine(m,m,m,e);
				}else{ // default to circle
					g.fillOval(d.width/4,d.width/4,d.width/2,d.height/2);
				}
			}
		}else{
			int rotate=0;
			Image img=null;
			Scriptable details=(Scriptable) this.details;
			String fname=details.get("file", details).toString();
			Object num=details.get("num", details);
			if(num instanceof Number)
				img=im.getImage(fname,((Number)num).intValue());
			Object rot=details.get("rotate", details);
			if(rot instanceof Number)
				rotate=((Number)rot).intValue()%4;

			if(rotate>0){
				Graphics2D g2=(Graphics2D)g;
				g2.rotate(Math.PI/2*rotate,
						d.getCenterX(),
						d.getCenterY());
			}
			
			g.drawImage(img, 0, 0, null);
		}
	}

	private int getSides() {
		int sides=0;
		
		if(details instanceof ScriptableObject 
				&& !(details instanceof Undefined)){
			Object n=ScriptableObject.getProperty((Scriptable) details, "north");
			Object s=ScriptableObject.getProperty((Scriptable) details, "south");
			Object e=ScriptableObject.getProperty((Scriptable) details, "east");
			Object w=ScriptableObject.getProperty((Scriptable) details, "west");
			
			Object ne=ScriptableObject.getProperty((Scriptable) details, "northeast");
			Object se=ScriptableObject.getProperty((Scriptable) details, "southeast");
			Object nw=ScriptableObject.getProperty((Scriptable) details, "northwest");
			Object sw=ScriptableObject.getProperty((Scriptable) details, "southwest");
			
			
			if(n instanceof Boolean && ((Boolean) n).booleanValue())sides+=N;
			if(s instanceof Boolean && ((Boolean) s).booleanValue())sides+=S;
			if(e instanceof Boolean && ((Boolean) e).booleanValue())sides+=E;
			if(w instanceof Boolean && ((Boolean) w).booleanValue())sides+=W;

			if(ne instanceof Boolean && ((Boolean) ne).booleanValue())sides+=NE;
			if(se instanceof Boolean && ((Boolean) se).booleanValue())sides+=SE;
			if(nw instanceof Boolean && ((Boolean) nw).booleanValue())sides+=NW;
			if(sw instanceof Boolean && ((Boolean) sw).booleanValue())sides+=SW;
}
		return sides;
	}

	private String findShape() {
		if(details instanceof ScriptableObject 
				&& !(details instanceof Undefined)){
			Object name=ScriptableObject.getProperty((Scriptable) details, "shape");
			String sname="circle";
			if(name!=null)sname=name.toString();
			return sname;
		}else return "circle";
	}

	private Color findColor() {
		if(details instanceof String){
			return Utils.getColor((String) details);
		}else if(details instanceof ScriptableObject 
				&& !(details instanceof Undefined)){
			Object name=ScriptableObject.getProperty((Scriptable) details, "color");
			return Utils.getColor(name);
		}else return Utils.getColor(null);
	}

	public boolean isColor() {
		if(details instanceof String)return true;
		if(details instanceof ScriptableObject
				&& !(details instanceof Undefined)){
			return ScriptableObject.hasProperty((Scriptable) details, "color");
		}
		return false;
	}

	public boolean isImage(){return !isColor();}

	public Object getDetails() {
		return details;
	}

	public void setDetails(Object details) {
		this.details = details;
	}
}
