package org.geekfu.hippo;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;

import javax.swing.JApplet;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import org.geekfu.MapPanel.MapPanel;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.Undefined;

public class HostGameWindow extends ScriptableObject implements ActionListener, WindowListener{
	private JFrame window;
	private JApplet applet;
	
	private MapPanel mapPanel;
	private HostMap hostMap;
	private HostSidebar sidebar;
	private JMenuBar menu;
	private boolean menuVisible=true;
	private boolean visible=true;
	
	public HostGameWindow(){}

	private static Integer numWindows=0;
	
	private String lastPath=null;
	
	public synchronized void jsConstructor(Object parent){
		if(parent==null || parent instanceof Undefined){
			JFrame window=new JFrame("Hippo!");
			window.setSize(830,640);
			parent=window;
		}
		
		mapPanel=new MapPanel(null,false,true);
		mapPanel.getTextEntry().requestFocus();
		mapPanel.addActionListener(new TextListener(this));

		synchronized(numWindows){numWindows++;}
		
		defineProperty("map",this.getClass(),ScriptableObject.PERMANENT);
		defineProperty("sidebar",this.getClass(),ScriptableObject.PERMANENT);
		defineProperty("textVisible",this.getClass(),ScriptableObject.PERMANENT);
		defineProperty("menuVisible",this.getClass(),ScriptableObject.PERMANENT);
		defineProperty("visible",this.getClass(),ScriptableObject.PERMANENT);
		defineFunctionProperties(new String[]{"setSize","print"}, this.getClass(), ScriptableObject.PERMANENT);

		menu=new JMenuBar();

		JMenu fileMenu=new JMenu("File");

		JMenuItem openJs=new JMenuItem("Load JS File...");
		openJs.setMnemonic('L');
		openJs.setActionCommand("loadJs");
		openJs.addActionListener(this);
		fileMenu.add(openJs);


		JMenuItem openBundle=new JMenuItem("Open Game Bundle...");
		openBundle.setMnemonic('O');
		openBundle.setActionCommand("openBundle");
		openBundle.addActionListener(this);
		openBundle.setEnabled(false);
		fileMenu.add(openBundle);

		JMenuItem reset=new JMenuItem("New Environment");
		reset.setMnemonic('N');
		reset.setActionCommand("newEnv");
		reset.addActionListener(this);
		fileMenu.add(reset);

		JMenuItem exit=new JMenuItem("Exit");
		exit.setMnemonic('Q');
		exit.setActionCommand("exit");
		exit.addActionListener(this);
		fileMenu.add(exit);

		menu.add(fileMenu);
		
		if(parent instanceof JFrame)
			setContainer((JFrame) parent);
		else if(parent instanceof JApplet)
			setContainer((JApplet) parent);
	}
	
	
	

	private void setContainer(JFrame window){
		this.applet=null;
		this.window=window;
		window.addWindowListener(this);
		window.setJMenuBar(menu);
		window.getContentPane().add(mapPanel);
		window.setVisible(true);
	}
	
	
	private void setContainer(JApplet applet){
		this.window=null;
		this.applet=applet;
		applet.getContentPane().add(mapPanel);
	}
	
	@Override
	public String getClassName() {
		return "GameWindow";
	}

	public HostMap getMap(){
		return hostMap;
	}
	public void setMap(Object map){
		if(map!=null && map instanceof HostMap){
			if(getMap()!=null)
				getMap().setWindow(null);
			hostMap=(HostMap) map;
			mapPanel.setMap(hostMap.getMap());
			hostMap.setWindow(this);
		}else{
			mapPanel.setMap(null);
		}
	}

	public HostSidebar getSidebar(){
		return sidebar;
	}
	public void setSidebar(Object obj){
		if(!(obj instanceof HostSidebar) && obj!=null){
			throw new EvaluatorException("Value must be a Sidebar");
		}
		HostSidebar sidebar=(HostSidebar) obj;
		if(this.sidebar!=null){
			this.sidebar.setPanel(null);
			this.sidebar.setWindow(null);
		}
		
		this.sidebar=sidebar;
		
		mapPanel.setSidebarVisible(sidebar!=null);
		if(sidebar!=null){
			sidebar.setPanel(mapPanel.getSidebar());
			sidebar.setWindow(this);
		}
	}

	public boolean getTextVisible(){
		return mapPanel.isText();
	}
	public void setTextVisible(boolean text){
		mapPanel.setText(text);
	}

	public void setSize(int width,int height){
		window.setSize(width,height);
	}

	public void print(Object obj){
		mapPanel.showString(obj.toString());
	}

	public MapPanel getMapPanel() {
		return mapPanel;
	}

	public boolean getMenuVisible() {
		return menuVisible;
	}

	public void setMenuVisible(boolean menuVisible) {
		this.menuVisible = menuVisible;
		if(this.menuVisible)
			window.setJMenuBar(menu);
		else window.setJMenuBar(null);
		window.getContentPane().invalidate();
		window.getContentPane().repaint();
	}

	public void actionPerformed(ActionEvent event) {
		if("loadJs".equals(event.getActionCommand())){
			if(lastPath==null)lastPath=System.getProperty("user.dir");
			JFileChooser chooser=new JFileChooser(lastPath);
			int result=chooser.showOpenDialog(window);
			if(result==JFileChooser.APPROVE_OPTION){
				try {
					File file=chooser.getSelectedFile();
					lastPath=file.getAbsolutePath();
					Context ctx=Utils.getContext();
					ctx.evaluateString(this, "include(\""+file.getAbsolutePath()+"\");", "", 0, null);
				} catch (Exception e) {
					mapPanel.showString("!! "+e.getMessage());
					e.printStackTrace();
				}
			}
		} else if("newEnv".equals(event.getActionCommand())){
			try {
				HippoMain.reset(Utils.getContext(),null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void windowClosing(WindowEvent arg0) {
		synchronized(numWindows){
			numWindows--;
			if(numWindows<1)System.exit(0);
		}
	}
	
	public void windowActivated(WindowEvent arg0) {}
	public void windowClosed(WindowEvent arg0) {}
	public void windowDeactivated(WindowEvent arg0) {}
	public void windowDeiconified(WindowEvent arg0) {}
	public void windowIconified(WindowEvent arg0) {}
	public void windowOpened(WindowEvent arg0) {}

	public boolean getVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
		window.setVisible(visible);
	}
}
