package org.geekfu.hippo;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;

import org.geekfu.OldCartographer.CellComponent;
import org.geekfu.OldCartographer.ColorCell;
import org.geekfu.OldCartographer.Map;
import org.geekfu.OldCartographer.MapCell;
import org.geekfu.hippo.Utils.GenericObject;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class HostMap extends ScriptableObject implements Notifiable, MouseListener{

	private Map map;
	private Function spaceFunction;

	private ScriptableObject spaceArray;

	private List<HostPiece>[][] pieces;
	private Function onclick=null; // Function

	private HostGameWindow window;

	public Function getOnclick() {
		return onclick;
	}

	public void setOnclick(Object onclick) {
		if(onclick instanceof Function)
			this.onclick = (Function) onclick;
		else this.onclick=null;
	}

	public void jsConstructor(int width, int height, int tilesize){
		if(width==0)width=1;
		if(height==0)height=1;
		if(tilesize==0)tilesize=48;

		defineProperty("width",this.getClass(),ScriptableObject.READONLY);
		defineProperty("height",this.getClass(),ScriptableObject.READONLY);
		defineProperty("cellsize",this.getClass(),ScriptableObject.PERMANENT);
		defineProperty("spaces",this.getClass(),ScriptableObject.PERMANENT);
		defineProperty("window",this.getClass(),ScriptableObject.READONLY);
		defineProperty("onclick",this.getClass(),0);
		defineFunctionProperties(new String[]{"repaint","place","remove","move","pieces","moveGroup"}, this.getClass(), ScriptableObject.READONLY);

		pieces=new List[width][height];
		
		MapCell cell=new ColorCell(Color.BLACK,Color.GREEN,true);
		map=new WrappedMap(width, height, tilesize);
		map.addMouseListener(this);
		map.setBorder(BorderFactory.createLineBorder(Color.black,5));
		map.revalidate();
		repaint();
	}

	@Override
	public String getClassName() {
		return "Map";
	}

	public Map getMap(){
		return map;
	}

	private class WrappedMap extends Map{
		public WrappedMap(int width, int height, int cellsize) {
			super(width, height, new Dimension(cellsize,cellsize));
		}
	}



	public ScriptableObject pieces(int x,int y){
		GenericObject ret=Utils.createObj(new PieceNotifiable(x,y));
		ret.putNoNotify("length", ret, "0");
		if(pieces[x][y]==null)
			return ret;
		else{
			int n=0;
			for(HostPiece p : pieces[x][y])
				ret.putNoNotify(n++, ret, p);
			ret.putNoNotify("length", ret, String.valueOf(pieces[x][y].size()));
			return ret;
		}
	}


	private class PieceNotifiable implements Notifiable{
		int x,y;

		public PieceNotifiable(int x, int y) {
			this.x = x;
			this.y = y;
		}

		public void notify(ScriptableObject src) {
			List<HostPiece> list=new ArrayList<HostPiece>();
			int n=0;
			Object obj=src.get(n, src);
			while(obj!=null && obj!=NOT_FOUND){
				if(obj instanceof HostPiece)list.add((HostPiece) obj);
				obj=src.get(++n, src);
			}
			if(pieces[x][y]!=null){
				clear(x,y);
			}
			for(int k=list.size()-1;k>=0;k--){
				place(list.get(k),x,y);
			}
		}

	}


	public void setSpaces(ScriptableObject spaces){
		if(spaces instanceof Function){
			spaceFunction=(Function) spaces;
			spaceArray=null;
		}else if(spaces != null){
			spaceArray=spaces;
			spaceFunction=null;
		}
		repaint();
	}

	public void clear(int x, int y) {
		if(pieces[x][y]!=null){
			for(HostPiece piece : pieces[x][y]){
				map.removePiece(piece.getCell());
			}
			pieces[x][y].clear();
			map.repaint();
		}
	}

	public Object getSpaces(){
		if(spaceFunction!=null)return spaceFunction;
		return spaceArray;
	}

	public void repaint() {
		if(spaceFunction!=null){
			Context ctx=Utils.getContext();
			for(int x=0;x<map.getMapWidth();x++){
				for(int y=0;y<map.getMapHeight();y++){
					Object ret=spaceFunction.call(ctx, this, this, new Object[]{x, y});
					map.putSpace(x, y, parseReturn(ret));
				}
			}
		}else if(spaceArray!=null){
			for(int x=0;x<map.getMapWidth();x++){
				Object col=ScriptableObject.getProperty(spaceArray,x);
				if(col instanceof ScriptableObject){
					for(int y=0;y<map.getMapHeight();y++){
						Object cell=ScriptableObject.getProperty((Scriptable) col,y);
						map.putSpace(x, y, parseReturn(cell));
					}
				}
			}
		}else{
			MapCell white=new ColorCell(Color.WHITE,null,false);
			MapCell gray=new ColorCell(Color.GRAY,null,false);
			spaceArray=Utils.createObj(this);

			for(int x=0;x<map.getMapWidth();x++){
				ScriptableObject col=Utils.createObj(this);
				((GenericObject) spaceArray).putNoNotify(x,spaceArray,col);

				MapCell start=(x%2==1?gray:white);

				for(int y=0;y<map.getMapHeight();y++){
					((GenericObject) col).putNoNotify(y, col, start==gray?"gray":"white");
					map.putSpace(x, y, start);
					start=(start==gray?white:gray);
				}
			}
		}

		map.repaint();
	}


	public MapCell parseReturn(Object ret){
		return new HippoCell(ret,true);
	}

	public int getWidth(){
		return map.getMapWidth();
	}


	public int getHeight(){
		return map.getMapHeight();
	}


	public int getCellsize(){
		return map.getCellsize().width;
	}

	public void setWidth(int width){
		throw new EvaluatorException("Cannot change the dimensions of a map after it's been made.");
	}


	public void setHeight(int height){
		throw new EvaluatorException("Cannot change the dimensions of a map after it's been made.");
	}

	public void setCellsize(int cellsize){
		if(cellsize<=0)
			throw new EvaluatorException("Cellsize must be at least 1.");
		map.setCellsize(new Dimension(cellsize,cellsize));
	}

	public void place(HostPiece piece,int x,int y){
		if(!map.inBounds(x, y))throw new EvaluatorException("("+x+", "+y+") is off the edge of the map.");
		if(piece==null)return;

		if(piece.getMap()!=null){
			piece.getMap().remove(piece);
		}
		
		map.putPiece(x, y, piece.getCell());
		
		piece.setMap(this);
		

		if(pieces[x][y]==null)
			pieces[x][y]=new ArrayList<HostPiece>();
		pieces[x][y].add(0, piece);
		piece.setPointNoNotify(x, y);
	}

	public void remove(HostPiece piece){
		if(piece.getMap()==this){
			map.removePiece(piece.getCell());
			map.repaint();
			pieces[(Integer) piece.getX()][(Integer) piece.getY()].remove(piece);
			piece.setMap(null);
		}else
			throw new EvaluatorException("Piece "+piece+" is not on this map.");
	}

	public void move(HostPiece piece,int endx,int endy,int duration,int steps){
		if(!map.inBounds(endx, endy))throw new EvaluatorException("("+endx+", "+endy+") is off the edge of the map.");
		if(duration==0)duration=250;
		if(steps==0)steps=10;

		if(piece.getMap()==this)
			pieces[(Integer) piece.getX()][(Integer) piece.getY()].remove(piece);
		else
			piece.setMap(this);

		if(pieces[endx][endy]==null)
			pieces[endx][endy]=new ArrayList<HostPiece>();
		pieces[endx][endy].add(0, piece);
		piece.setPointNoNotify(endx,endy);

		if(piece.getMoving()==null){
			Thread thread=map.animateMove(piece, 
					endx, 
					endy, 
					duration, 
					steps);
			piece.setMoving(thread);
			thread.start();
		} else {
			throw new EvaluatorException("Piece is already being moved. Use the onanimate handler to wait until it finishes.");
		}
	}


	public synchronized void moveGroup(ScriptableObject group, int dx, int dy, int duration, int steps){
		List<HostPiece> pieces=new ArrayList<HostPiece>();
		Object len=group.get("length", group);
		if(len==null || !(len instanceof Number))
			throw new EvaluatorException("Group must be an array (define 'length' property)");
		int length=((Number) len).intValue();

		for(int n=0;n<length;n++){
			Object obj=group.get(n, group);
			if(obj instanceof HostPiece)
				pieces.add((HostPiece) obj);
		}

		List<HostPiece> real=new ArrayList<HostPiece>();

		for(HostPiece p : pieces){
			if(p.getMoving()==null)
				real.add(p);
		}
		pieces=real;

		if(pieces.isEmpty())return;

		if(duration==0)duration=250;
		if(steps==0)steps=10;

		for(HostPiece piece : pieces){
			int cx=(piece.getX()==null?0:((Integer) piece.getX()).intValue());
			int cy=(piece.getY()==null?0:((Integer) piece.getY()).intValue());
			int endx=cx+dx,endy=cy+dy;

			if(this.pieces[endx][endy]==null)
				this.pieces[endx][endy]=new ArrayList<HostPiece>();

			if(piece.getMap()==this)
				this.pieces[cx][cy].remove(piece);
			else
				piece.setMap(this);
		}

		MultiMoveThread t=new MultiMoveThread(pieces,this,dx,dy,duration,steps);
		for(HostPiece p : pieces)p.setMoving(t);
		t.start();
	}


	public synchronized void notify(ScriptableObject src) {
		repaint();
	}

	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	public void mousePressed(MouseEvent e) {
		try{
			int x=-1,y=-1;
			if(e.getSource() instanceof CellComponent){
				x=((CellComponent) e.getSource()).getMapX();
				y=((CellComponent) e.getSource()).getMapY();
			}
			if(!map.inBounds(x,y))return; // this'll likely never happen

			int btn=(e.getButton()==MouseEvent.BUTTON1?1:
				e.getButton()==MouseEvent.BUTTON2?2:
					e.getButton()==MouseEvent.BUTTON3?3:4);
			Object[] args=new Object[]{x, y, btn};

			boolean skipMap=false;
			List<HostPiece> pieces=this.pieces[x][y];
			if(pieces!=null)
				for(HostPiece p : pieces)
					if(p.getOnclick()!=null){
						Context ctx=Utils.getContext();
						Object next=p.getOnclick().call(ctx, p.getOnclick().getParentScope(), p, args);
						if(next.equals(Boolean.TRUE)){
							skipMap=true;
							break;
						}
					}

			if(onclick!=null && !skipMap){
				Context ctx=Utils.getContext();
				getOnclick().call(ctx, onclick.getParentScope(), this, args);
			}
		}catch(Exception exc){
			if(getWindow()!=null)
				getWindow().getMapPanel().showString("!! "+exc.getMessage());
		}
	}

	public HostGameWindow getWindow() {
		return window;
	}

	public void setWindow(HostGameWindow window) {
		this.window = window;
	}

	public List<HostPiece>[][] getPieces() {
		return pieces;
	}

}
