package org.geekfu.hippo;

import org.geekfu.OldCartographer.MapCell;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class HostPiece extends ScriptableObject{

	private HippoCell cell;
	private HostMap map;
	private int tilenum;
	private int x,y;
	
	private Function onclick;
	private Object details;
	private Thread moveThread=null;
	private Function onanimate;
	
	public void repaint(){
		if(map!=null)map.getMap().repaintCell(cell);
	}
	
	public Object getDetails() {
		return details;
	}

	public void setDetails(Object details) {
		this.details = details;
		cell.setDetails(details);
		if(map!=null)map.repaint();
	}

	public int getTilenum() {
		return tilenum;
	}

	public void setTilenum(int tilenum) {
		this.tilenum = tilenum;
	}

	@Override
	public String getClassName() {
		return "Piece";
	}

	public void jsConstructor(Object param){
		defineProperty("map", this.getClass(), ScriptableObject.READONLY);
		defineProperty("x", this.getClass(), ScriptableObject.PERMANENT);
		defineProperty("y", this.getClass(), ScriptableObject.PERMANENT);
		defineProperty("onclick", this.getClass(), ScriptableObject.PERMANENT);
		defineProperty("onanimate", this.getClass(), ScriptableObject.PERMANENT);
		defineProperty("details", this.getClass(), ScriptableObject.PERMANENT);
		defineFunctionProperties(new String[]{"move","repaint"}, this.getClass(), ScriptableObject.READONLY);
		cell=new HippoCell(param);
		this.details=param;
	}

	public MapCell getCell() {
		return cell;
	}

	public HostMap getMap() {
		return map;
	}

	public void setMap(HostMap newmap) {
		if(newmap!=map && map!=null && newmap!=null)
			map.remove(this);
		
		this.map = newmap;
	}
	
	public String toString(){
		if(details instanceof String)
			return super.toString()+"("+details+")";
		else{
			String file=((ScriptableObject) details).get("file",(Scriptable) details).toString();
			String num=((ScriptableObject) details).get("num",(Scriptable) details).toString();
			return super.toString()+"("+file+":"+num+")";
		}
	}
	
	public Object getX(){
		if(map==null)return null;
		return x;
	}
	
	public Object getY(){
		if(map==null)return null;
		return y;
	}
	
	public void setX(int x){
		this.x=x;
		if(map!=null)map.place(this, x, y);
	}
	
	public void setY(int y){
		this.y=y;
		if(map!=null)map.place(this, x, y);
	}
	
	public void setPointNoNotify(int x,int y){
		this.x=x;this.y=y;
	}
	
	public void move(int endx,int endy,int duration,int steps){
		if(map==null)
			throw new EvaluatorException("This piece is not on a map");
		map.move(this, endx, endy, duration, steps);
	}
	
	
	public Function getOnclick() {
		return onclick;
	}

	public void setOnclick(Function onclick) {
		this.onclick = onclick;
	}

	public Thread getMoving() {
		return moveThread;
	}

	public void setMoving(Thread moving) {
		Thread old=moveThread;
		this.moveThread = moving;

		if(moving!=old && moving!=null && onanimate!=null){
			onanimate.call(Utils.getContext(), onanimate.getParentScope(), this, new Object[]{"start"});
		}
		
		if(moving==null && onanimate!=null){
			onanimate.call(Utils.getContext(), onanimate.getParentScope(), this, new Object[]{"end"});
		}
	}

	public Function getOnanimate() {
		return onanimate;
	}

	public void setOnanimate(Function onanimate) {
		this.onanimate = onanimate;
	}
}
