package org.geekfu.hippo;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import org.geekfu.MapPanel.CenteredPanel;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class HostSidebar extends ScriptableObject{

	private JPanel panel;
	private HostGameWindow window;

	public HostSidebar(){
	}

	@Override
	public String getClassName() {
		return "Sidebar";
	}

	public void jsConstructor(){
		defineFunctionProperties(new String[]{"repaint"}, this.getClass(), ScriptableObject.PERMANENT);
	}

	@Override
	public void put(int idx, Scriptable scope, Object obj) {
		super.put(idx,scope,obj);
		repaint();
	}

	private JComponent makeComponent(Object obj){
		if(obj==null){
			return new JPanel();
		}else if(obj instanceof String){
			return new JLabel(obj.toString());
		}else if(obj instanceof ScriptableObject){
			if(obj instanceof HostMap){
				HostMap map=(HostMap)obj;
				map.setWindow(getWindow());
				JScrollPane mapscroll=new JScrollPane(new CenteredPanel(map.getMap()));
				if(map!=null){
					mapscroll.getVerticalScrollBar().setUnitIncrement(map.getMap().getCellsize().height/2);
					mapscroll.getHorizontalScrollBar().setUnitIncrement(map.getMap().getCellsize().width/2);
				}
				return mapscroll;
			}else if(obj instanceof HostItemList){
				HostItemList itemList=(HostItemList) obj;
				JComponent comp=new JScrollPane(itemList.getItemList());
				comp.setPreferredSize(new Dimension(5000,100));
				return comp;
			}

			ScriptableObject so=(ScriptableObject) obj;
			Object typeObj=ScriptableObject.getProperty(so, "type");
			if(typeObj!=null){
				String type=typeObj.toString();
				if(type.equals("button")){
					String label=ScriptableObject.hasProperty(so, "label")?
							ScriptableObject.getProperty(so, "label").toString():
								"";
							return new JSButton(so,label);
				}else if(type.equals("line")){
					JPanel line=new JPanel();
					line.setPreferredSize(new Dimension(5000,3));
					Object color=ScriptableObject.getProperty(so, "color");
					if(color==null)color="black";
					line.setBackground(Utils.getColor(color.toString()));
					return line;
				}else if(type.equals("space")){
					JPanel sep=new JPanel();
					return sep;
				}else if(type.equals("input")){
					return new JSInput(so);
				}
			}
		}
		JPanel comp=new JPanel();
		comp.setBackground(Color.red);
		return comp;
	}

	public void repaint(){
		if(panel==null)return;

		panel.removeAll();
		TableLayout tbl=new TableLayout();
		panel.setLayout(tbl);

		tbl.insertColumn(0, TableLayout.FILL);
		int rows=0;

		Object[] ids=getIds();

		for(Object id : ids){
			if(!(id instanceof Integer))continue;
			int idx=((Integer) id).intValue();

			Object obj=get(idx, getParentScope());
			String type="";
			if(obj instanceof ScriptableObject){
				Object o=ScriptableObject.getProperty((Scriptable) obj, "type");
				if(o instanceof String)type=o.toString();
			}
			JComponent comp=makeComponent(obj);

			tbl.insertRow(rows, TableLayout.PREFERRED);

			if(type.equals("space") || comp instanceof JScrollPane)
				tbl.setRow(rows, TableLayout.FILL);

			panel.add(comp,"0, "+rows+", f, f");

			if(comp instanceof JSInput)((JSInput)comp).refreshValue();
			rows++;
		}
		panel.validate();
		panel.repaint();
	}

	public JPanel getPanel() {
		return panel;
	}

	public void setPanel(JPanel panel) {
		this.panel = panel;
		repaint();
	}

	private class JSButton extends JButton implements ActionListener{
		private ScriptableObject params;
		public JSButton(ScriptableObject params,String label){
			super(label);
			this.params=params;
			addActionListener(this);
		}
		public void actionPerformed(ActionEvent event) {
			Object obj=ScriptableObject.getProperty(params, "onclick");
			if(obj instanceof Function){
				Function fn=(Function) obj;
				Context ctx=Utils.getContext();
				Scriptable scope=fn.getParentScope();
				try{
					fn.call(ctx, scope, params, new Object[]{});
				}catch(Exception exc){
					if(getWindow()!=null)
						getWindow().getMapPanel().showString("!! "+exc.getMessage());
				}
			}else{
				throw new EvaluatorException("onclick must be a function.");
			}
		}
	}

	private class JSInput extends JTextField implements ActionListener,KeyListener{
		private ScriptableObject params;

		public JSInput(ScriptableObject params){
			this.params=params;
			addActionListener(this);
			addKeyListener(this);

			refreshValue();
		}

		private void refreshValue() {
			Object str=ScriptableObject.getProperty(params, "value");
			if(str!=null)setText(str.toString());
		}

		public void actionPerformed(ActionEvent arg0) {
			params.put("value", params, this.getText());
			Object obj=ScriptableObject.getProperty(params, "onaction");
			if(obj instanceof Function){
				Function fn=(Function) obj;
				Context ctx=Utils.getContext();
				Scriptable scope=fn.getParentScope();
				try{
					fn.call(ctx, scope, params, new Object[]{this.getText()});
				}catch(Exception exc){
					if(getWindow()!=null)
						getWindow().getMapPanel().showString("!! "+exc.getMessage());
				}
			}else{
				throw new EvaluatorException("onaction must be a function.");
			}
		}

		public void keyTyped(KeyEvent event) {
			params.put("value", params, this.getText());
		}
		public void keyReleased(KeyEvent event) {
			params.put("value", params, this.getText());
		}
		public void keyPressed(KeyEvent event) {
			params.put("value", params, this.getText());
		}
	}

	public HostGameWindow getWindow() {
		return window;
	}

	public void setWindow(HostGameWindow window) {
		this.window = window;
	}
}