package org.geekfu.hippo;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import java.util.LinkedList;
import java.util.Random;

import javax.swing.JApplet;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.ScriptableObject;

public class HostSystem extends ScriptableObject {

	private static LinkedList<String> paths=new LinkedList<String>();
	private static Random rng=new Random();

	@Override
	public String getClassName() {
		return "System";
	}

	public void jsConstructor(){
		defineFunctionProperties(new String[]{"include"}, this.getClass(), ScriptableObject.READONLY);
		defineFunctionProperties(new String[]{"loadImage"}, this.getClass(), ScriptableObject.READONLY);
		defineFunctionProperties(new String[]{"random"}, this.getClass(), ScriptableObject.READONLY);
	}

	public void include(String path){
		try {
			if(!HippoMain.isApplet()){
				File file=new File(path);
				if(!file.isAbsolute()){
					if(!paths.isEmpty())
						file=new File(paths.getFirst(),path);
				}
				if(!file.exists()){
					throw new EvaluatorException("File "+file.getPath()+" not found.");
				}

				paths.addFirst(file.getParent());

				Context ctx=Utils.getContext();

				Reader reader = new FileReader(file);
				ctx.evaluateReader(getParentScope(), reader, "", 1, null);
				paths.removeFirst();
			} else {
				JApplet applet=HippoMain.getApplet();
				
				URL file=new URL(applet.getCodeBase()+"/"+path);
				
				paths.addFirst(path);
				
				URLConnection conn=file.openConnection();
				conn.setDoInput(true);
				conn.setUseCaches(false);

				Context ctx=Utils.getContext();

				Reader reader=new InputStreamReader(conn.getInputStream());
				ctx.evaluateReader(getParentScope(), reader, "", 1, null);
				paths.removeFirst();			
			}
		} catch (Exception e) {
			throw new EvaluatorException(e.getMessage());
		}

	}

	public String loadImage(String path, int dimension){
		if(dimension<1)
			throw new EvaluatorException("Must specify a tile size of at least 1");

		File file=new File(path);
		if(!file.isAbsolute()){
			if(!paths.isEmpty()){
				if(HippoMain.isApplet()){
					String top=paths.getFirst();
					file=new File(top.substring(0,top.lastIndexOf("/")),path);
				}else{
					file=new File(paths.getFirst(),path);
				}
			}
		}
		if(!HippoMain.isApplet() && !file.exists()){
			throw new EvaluatorException("Image file "+file.getPath()+" not found.");
		}

		ImageManager im=ImageManager.instance();

		try {
			return im.loadImage(file.getPath(),dimension);
		} catch (IOException e) {
			throw new EvaluatorException(e.getMessage());
		}
	}

	public int random(int max){
		if(max<0)return rng.nextInt();
		return rng.nextInt(max);
	}
}
