package org.geekfu.hippo;

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.swing.JApplet;

public class ImageManager {
	private static ImageManager _instance;
	public static ImageManager instance(){
		if(_instance==null)_instance=new ImageManager();
		return _instance;
	}

	private ImageManager(){
		NOT_FOUND=new BufferedImage(1,1,BufferedImage.TYPE_INT_RGB);
		NOT_FOUND.getGraphics().fillRect(0, 0, 1, 1);
	}

	private Map<String,ImageData> images=new HashMap<String,ImageData>();
	private static Image NOT_FOUND=null;

	private class ImageData{
		int dimension;
		int maxx,maxy;
		Image[] cells;
		Image image;
	}

	public String loadImage(String file,int dimension) throws IOException{
		Image img=null;

		if(HippoMain.isApplet()){
			System.out.println("is applet");
			JApplet applet=HippoMain.getApplet();
			String base=applet.getCodeBase().toString();
			if(!base.endsWith("/"))base+="/";
			file=file.replace("\\", "/");
			URL url=new URL(base+file);
			System.out.println("url: "+url);
			URLConnection conn=url.openConnection();
			conn.setDoInput(true);
			conn.setUseCaches(false);

			img=ImageIO.read(conn.getInputStream());
		}else{
			img=ImageIO.read(new File(file));
		}
		
		ImageData data=new ImageData();

		while(img.getWidth(null)==-1); // spin until loaded

		data.image=img;
		data.dimension=dimension;
		data.maxx=img.getWidth(null)/dimension;
		data.maxy=img.getHeight(null)/dimension;

		data.cells=new Image[data.maxx*data.maxy];

		String name=Utils.unique(file,images.keySet());
		images.put(name, data);

		return name;
	}

	public Image getImage(String key,int number){
		if(images.containsKey(key))
			return getCell(images.get(key),number);
		else return NOT_FOUND;
	}

	private Image getCell(ImageData data, int number){
		if(number>data.maxx*data.maxy)return NOT_FOUND;
		int x=(data.maxx>0?number%data.maxx:0);
		int y=(data.maxx>0?number/data.maxx:0);
		if(data.cells[x+y*data.maxx]==null){
			data.cells[x+y*data.maxx]=((BufferedImage) data.image).getSubimage(
					x*data.dimension,
					y*data.dimension,
					data.dimension,
					data.dimension);
		}
		return data.cells[x+y*data.maxx];
	}
}
