package org.geekfu.hippo;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashMap;
import java.util.TreeMap;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.geekfu.OldCartographer.Map;
import org.geekfu.OldCartographer.MapCell;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.Undefined;

public class ItemList extends JPanel implements MouseListener{

	private class Item{
		public String label;
		public MapCell piece;
		public Map map;
		public Color foreground,background;
		public Function onclick;
		int row;
	}
	
	private TreeMap<Integer,ScriptableObject> items;
	private HashMap<Object,Item> rows;
	private int cellsize=48;
	private TableLayout layout;
	
	public ItemList(){
		items=new TreeMap<Integer,ScriptableObject>();
		rows=new HashMap<Object,Item>();
		setLayout(layout=new TableLayout());
		layout.insertColumn(0, TableLayout.FILL);
	}

	private Item makeItem(ScriptableObject obj){
		Item item=new Item();
		Object label=ScriptableObject.getProperty(obj, "label");
		Object piece=ScriptableObject.getProperty(obj, "piece");
		
		if(label==null)label="";
		if(piece==null || !(piece instanceof HostPiece)){
			piece=new HostPiece();
			((HostPiece)piece).setDetails("black");
		}
		
		item.label=label.toString();
		item.piece=((HostPiece)piece).getCell();
		item.map=new Map(1,1,new Dimension(cellsize,cellsize));
		item.map.putSpace(0, 0, item.piece);
		
		Object fg=null,bg=null;
		if(ScriptableObject.hasProperty(obj, "foreground"))
			fg=ScriptableObject.getProperty(obj, "foreground");
		if(ScriptableObject.hasProperty(obj, "background"))
			bg=ScriptableObject.getProperty(obj, "background");
		if(fg!=null && !(fg instanceof Undefined))
			item.foreground=Utils.getColor(fg);
		if(bg!=null && !(bg instanceof Undefined))
			item.background=Utils.getColor(bg);
		
		if(ScriptableObject.getProperty(obj, "onclick") instanceof Function)
			item.onclick=(Function) ScriptableObject.getProperty(obj, "onclick");
		
		return item;
	}
	
	public void putItem(int idx,ScriptableObject obj){
		items.put(idx,obj);
	}
	
	public void removeItem(int idx){
		items.remove(idx);
	}
	
	public ScriptableObject getItem(int idx){
		return items.get(idx);
	}
	
	public void refresh(){
		this.removeAll();
		rows.clear();
		int cnt=0;
		
		for(int idx : items.keySet()){
			if(idx>=layout.getNumRow())
				layout.insertRow(cnt, TableLayout.PREFERRED);
			
			Item item=makeItem(items.get(idx));
			JLabel lbl=new JLabel(item.label);
			
			JPanel pan=new JPanel();
			pan.setLayout(new TableLayout(new double[][]{
					{TableLayout.PREFERRED,5,TableLayout.FILL},
					{TableLayout.PREFERRED}
			}));
			
			if(item.foreground!=null)
				lbl.setForeground(item.foreground);
			if(item.background!=null)
				pan.setBackground(item.background);

			pan.add(item.map,"0, 0");
			pan.add(lbl,"2, 0");
			
			if(item.onclick!=null){
				pan.addMouseListener(this);
				item.map.addMouseListener(this);
				item.row=idx;
				rows.put(pan, item);
				rows.put(item.map.getComponent(0,0), item);
			}
			
			add(pan,"0, "+cnt);
			cnt++;
		}
		validate();
		repaint();
	}
	
	public int getCellsize() {
		return cellsize;
	}

	public void setCellsize(int cellsize) {
		this.cellsize = cellsize;
	}
	
	public int getLength(){
		return items.size();
	}
	
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	public void mousePressed(MouseEvent e) {
		Item it=getItemForSource(e.getSource());
		it.onclick.call(Utils.getContext(), it.onclick.getParentScope(), null, new Object[]{it.row});
	}

	private Item getItemForSource(Object obj){
		return rows.get(obj);
	}
}
