package org.geekfu.hippo;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.geekfu.OldCartographer.Map;
import org.geekfu.OldCartographer.MapLayoutConstraints;
import org.mozilla.javascript.EvaluatorException;

public class MultiMoveThread extends Thread{
	Collection<HostPiece> pieces;
	Map map;
	int dx,dy,duration,steps;
	HostMap hostMap;

	public MultiMoveThread(Collection<HostPiece> pieces,HostMap hostMap,int dx,int dy,int duration,int steps){
		this.pieces=pieces;
		this.map=hostMap.getMap();
		this.hostMap=hostMap;
		this.dx=dx;
		this.dy=dy;
		this.duration=duration;
		this.steps=steps;
	}

	public void run(){
		if(pieces.isEmpty())return;

		for(HostPiece p : pieces){
			int cx=(p.getX()==null?0:((Integer) p.getX()).intValue());
			int cy=(p.getY()==null?0:((Integer) p.getY()).intValue());
			int endx=dx+cx,endy=dy+cy;

			if(!map.inBounds(endx, endy))
				throw new EvaluatorException("A piece at ("+cx+", "+cy+") would end off the edge of the map.");
			
			hostMap.getPieces()[endx][endy].add(0, p);
			
			p.setPointNoNotify(((Integer) p.getX()).intValue()+dx,((Integer) p.getY()).intValue()+dy);
		}


		double deltax=(double)(dx)/(double)(steps)*map.getCellsize().width;
		double deltay=(double)(dy)/(double)(steps)*map.getCellsize().height;

		double[] currentx=new double[pieces.size()];
		double[] currenty=new double[pieces.size()];

		List<MapLayoutConstraints> constraints=new ArrayList<MapLayoutConstraints>();
		for(HostPiece p : pieces){
			if(p.getCell().getConstraints()==null)
				map.putPiece(0,0, p.getCell());
			constraints.add(p.getCell().getConstraints());
		}

		int k=0;
		for(MapLayoutConstraints cons : constraints){
			Point pt=map.getOrigin(cons);
			currentx[k]=0;
			currenty[k++]=0;
		}

		try {				
			for(int n=0;n!=steps;n++){
				int j=0;
				for(MapLayoutConstraints cons : constraints){
					currentx[j]+=deltax;
					currenty[j]+=deltay;

					cons.offset.x=(int) currentx[j];
					cons.offset.y=(int) currenty[j];

					j++;
				}
				map.revalidate();
				sleep(duration/steps);
			}
		}catch(InterruptedException e){}
		finally{
			for(HostPiece p : pieces){
				int x=((Integer) p.getX()).intValue();
				int y=((Integer) p.getY()).intValue();
				p.getCell().setConstraints(new MapLayoutConstraints(x+dx,y+dy,0,0));
				map.putPiece(x,y,p.getCell());
			}
			map.revalidate();
			for(HostPiece p : pieces)
				p.setMoving(null);
		}

	}
}
