package org.geekfu.Cartographer;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Vector;

public class IsoMap extends Map{
	
	@Override
	public MoveThread animateMove(MapCell cell, MapLayoutConstraints end, long duration, int steps) {
		// TODO Auto-generated method stub
		if(end.point.x>=cell.getConstraints().point.x && 
				end.point.y<=cell.getConstraints().point.y)
			movePiece(cell,cell.getConstraints().point.x,
					cell.getConstraints().point.y,
					end.point.x,
					end.point.y);
		return super.animateMove(cell, end, duration, steps);
	}
	
	public IsoMap(int width, int height, Dimension cellsize) {
		super(width, height, cellsize);
	}
	
	public IsoMap(int width, int height, MapCell cell, Dimension cellsize) {
		super(width, height, cell, cellsize);
	}
	
	public IsoMap(int width, int height) {
		super(width, height);
	}
	
	@Override
	public Dimension getPreferredSize() {
		int max=getMapWidth()>getMapHeight()?getMapWidth():getMapHeight();
		return new Dimension(
				(getMapHeight()+getMapWidth())*getCellsize().width/2,
				(getMapHeight()+getMapWidth()+2)*getCellsize().width/4
		);
	}
	
	private Graphics getLocalGraphics(int x,int y,int xOff,int yOff,Graphics g){
		Point origin=getOrigin(new MapLayoutConstraints(x,y,xOff,yOff));
		Graphics local=g.create(origin.x,origin.y,getCellsize().width,getCellsize().height);
		return local;
	}
	
	@Override
	public void paintComponent(Graphics screen) {
		long startTime=System.currentTimeMillis();
		Point origin;
		MapCell current;
		
		Graphics g=buffer.getGraphics();
		Rectangle bounding=new Rectangle();
		
		synchronized(pieces)
		{
			synchronized(dirty)
			{
			for(Rectangle r : dirty)
				bounding=bounding.union(r);
			g.setClip(bounding);
			dirty.clear();
			}
						
			g.setColor(java.awt.Color.WHITE);
			g.fillRect(bounding.x,bounding.y,bounding.width,bounding.height);
			for(int y=0;y!=getMapHeight();y++)
			{
				for(int x=0;x!=getMapWidth();x++)
				{
					current=getSpace(x,y);
					origin=getOrigin(current.getConstraints());
					g.drawImage(current.getCanvas(),origin.x,origin.y,null);
				}
			}
			
			for(int y=0;y!=getMapHeight();y++)
				for(int x=0;x!=getMapWidth();x++)
				{
					current=getPiece(x,y);
					if(current!=null)
					{
						origin=getOrigin(current.getConstraints());
						g.drawImage(current.getCanvas(),origin.x,origin.y,null);
					}
				}
		}
		
		screen.drawImage(buffer,0,0,null);
		System.out.println("time: "+(System.currentTimeMillis()-startTime));
	}
	
	private void drawPieces(Graphics g, int x, int y) {
		Vector<MapCell> stack=getPieces(x,y);
		Point origin;
		
		for(int n=0;n!=stack.size();n++)
			for(MapCell piece : getPieces(x,y))
			{
				origin=getOrigin(piece.getConstraints());
				g.drawImage(piece.getCanvas(),origin.x,origin.y,null);
			}
	}
	
	@Override
	protected MapCell findCurrentCell(int x, int y) {
		int yc=(y+(getPreferredSize().width-x)/2);
		int xc=(y+x/2);
		yc-=getMapWidth()*getCellsize().width/4;
		xc-=getMapHeight()*getCellsize().width/4;
		
		xc/=(getCellsize().width/2);
		yc/=(getCellsize().width/2);
		
		xc--;yc--;
		
		if(!inBounds(xc,yc))return null;
		return cells[xc][yc];
	}
	
	@Override
	public Point getOrigin(MapLayoutConstraints loc) {
		int xc=(getMapHeight()-loc.point.y-1+loc.point.x)*(getCellsize().width/2);
		int yc=(loc.point.x+loc.point.y)*(getCellsize().width/4);
		xc+=loc.offset.x;
		yc+=loc.offset.y;
		return new Point(xc,yc);
	}
}
